<?php

/**
 * This software is intended for use with Oxwall Free Community Software http://www.oxwall.org/ and is a proprietary licensed product. 
 * For more information see License.txt in the plugin folder.

 * ---
 * Copyright (c) 2012, Purusothaman Ramanujam
 * All rights reserved.

 * Redistribution and use in source and binary forms, with or without modification, are not permitted provided.

 * This plugin should be bought from the developer by paying money to PayPal account (purushoth.r@gmail.com).

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
class BOL_BackupService {

    private static $classInstance;

    public static function getInstance() {
        if (self::$classInstance === null) {
            self::$classInstance = new self();
        }

        return self::$classInstance;
    }

    private function __construct() {
        
    }

    public function getAllTableNames() {
        return OW::getDbo()->queryForColumnList('SHOW TABLES');
    }

    function backupTables($tables) {
        $db = OW::getDbo();

        $return = "";

        foreach ($tables as $table) {
            $result = $db->queryForList('SELECT * FROM ' . $table);

            // First part of the output – remove the table
            $return .= 'DROP TABLE ' . $table . ';';

            // Second part of the output – create table
            $row2 = $db->queryForRow('SHOW CREATE TABLE ' . $table);
            list(,$createTableSql) = array_values($row2);
            $return .= "\n\n" . $createTableSql . ";\n\n";

            // Third part of the output – insert values into new table
            if (count($result) > 0) {
                $num_fields = count($result[0]);
                foreach ($result as $row) {
                    $return.= 'INSERT INTO ' . $table . ' VALUES(';

                    $j=0;foreach ($row as $column => $value) {
                        if (isset($value)) {
                            $value = addslashes($value);
                            $value = str_replace("\n", "\\n", $value);

                            $return .= '"' . $value . '"';
                        } else {
                            $return .= '""';
                        }
                        if ($j++ < ($num_fields - 1)) {
                            $return.= ',';
                        }
                    }
                    $return.= ");\n";
                }
            }
            $return.="\n\n\n";
        }

        $path = $this->getBackupPath();
        $file = OW_DB_NAME . '_dump_' . date("d.m.Y_H:i:s") . '.sql';

        $filess = $path . DS . $file;

        $handle = fopen($filess, 'w+');
        fwrite($handle, $return);
        fclose($handle);
    }

    public function zipFolders() {
        require_once( OW_DIR_LIB . 'pclzip.lib.php');

        $zipPath = $this->getBackupPath();        
        $file = 'backup_' . date("d.m.Y_H:i:s") . '.zip';
        $filess = $zipPath . DS . $file;

        $archive = new PclZip($filess);

        $v_dir = OW_DIR_ROOT;
        $v_remove = $v_dir;

        $v_list = $archive->create($v_dir, PCLZIP_OPT_REMOVE_PATH, $v_remove);
        if ($v_list == 0) {
            die("Error : " . $archive->errorInfo(true));
        }
    }

    public function backupProcess($tabs) {
        ini_set("memory_limit", "60M");
        ini_set("max_execution_time", "3600");

        $this->backupTables($tabs);
        $this->zipFolders();
    }

    public function getBackupPath() {
        $path = dirname(OW_DIR_ROOT).DS.'ow_backup'.DS;
        if (!file_exists($path)) {
            mkdir($path);
        }
        return $path;
    }

}